@@ -29,10 +29,6 @@ int screen_cols;
 
 static int cursor_visibility = -1;
 
-static void widget_handle_key(int key)
-{
-}
-
 static void update_cursor_visibility(void)
 {
 	const struct widget *active_widget;
@@ -87,9 +83,6 @@ void widget_init(struct widget *widget, int lines_, int cols, int y, int x,
 		set_panel_userptr(widget->panel, widget);
 	}
 
-	//if (!widget->handle_key)
-	//	widget->handle_key = widget_handle_key;
-
 	if (old_window)
 		delwin(old_window);
 
